#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé juin 2025
@auteur: david ALBERTO (www.astrolabe-science.fr)
Déplacement d'étoiles par rapport au pôle céleste nord, au fil
des millénaires.
"""
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from skyfield.api import Star, load
from skyfield.data import hipparcos
from skyfield.api import Loader #ajuster le fichier de téléchargement d'éphémérides

plt.rcParams["font.family"] = "Roboto Condensed"
plt.rcParams["ytick.color"] = "white"

# =============================================================================
# # Paramètres réglables par l'utilisateur :
# =============================================================================
taille_points = 100
pasTemps = 1000  # en années
annees = range(-12000, 15000, pasTemps)
#  mise en forme commune des points
prop_points = dict(marker='*', s=taille_points, ec='midnightblue', lw=0.2)
couleur_polaris = 'orange'
couleur_vega = 'lightcyan'
# -----------------------------------------------

# =============================================================================
# Définition des étoiles :
# =============================================================================
#Chargement des coordonnées du catalogue Hipparcos :
with load.open(hipparcos.URL) as f:
    df = hipparcos.load_dataframe(f)

# Rechercher le numéro d'une étoile dans le catalogue Hipparcos :
    # https://vizier.u-strasbg.fr/viz-bin/VizieR?-source=1239/hip_main
# Insérer ce numéro entre crochets :
polaris = Star.from_dataframe(df.loc[11767])
vega = Star.from_dataframe(df.loc[91262])

# -----------------------------------
load = Loader('../skyfield-ephemerides')
planets = load('de431t.bsp')  # 3.1GB fichier d'éphémérides (-13200 to 17191)

# lieu d'observation :
earth = planets['earth']
# =============================================================================
# TEMPS
# =============================================================================
ts = load.timescale()
periode = ts.utc(annees)
#
# =============================================================================
# # PARAMETRES DU GRAPHIQUE :
# =============================================================================
fig = plt.figure(figsize=(8,8),tight_layout=True, facecolor='lavender')
ax = plt.subplot(111,projection='polar', facecolor='midnightblue')
# ax.set_xticks([])
titre = f"Précession des équinoxes et déplacement d'étoiles\n de {annees[0]} à {annees[-1]} tous les {pasTemps} ans"
ax.set_title(titre, fontsize=14,
             fontweight = 'bold')


# =============================================================================
# FONCTIONS
# =============================================================================

def startrack(star, name, color):
    """
    Trace les positions d'une étoile durant la période.
    star : astre Skyfield
    name : string
    color : string
    """
    equinoxe = ts.J(annees)# prise en compte de la précession et de la nutation
    position = earth.at(periode).observe(star)
    ra, dec, distance = position.radec(equinoxe)
    X = np.radians(ra.hours*15)
    Y = 90 - dec.degrees
    ax.scatter(X, Y, c=color, **prop_points, label=name)
    return X, Y

def fleche(annee, xstar, ystar, xpos, ypos, color):
    """
    annote l'année sur la position.
    """
    idx = annees.index(annee)
    x0 = xstar[idx]
    y0 = ystar[idx]
    ax.annotate(annee, (x0, y0), c=color, xytext=(np.radians(xpos), ypos),
                arrowprops=dict(color=color,
                                 linewidth=1,
                                  arrowstyle='->',
                shrinkA=2, shrinkB=5,
                ))


# =============================================================================
# #  Tracé :
# =============================================================================


xpol, ypol = startrack(polaris, 'Polaris', couleur_polaris)
xveg, yveg = startrack(vega, 'Vega', couleur_vega)

#  flèches d'annotation pour Polaris :
fleche(-10000, xpol, ypol, 270, 38, couleur_polaris)
fleche(-6000, xpol, ypol, 300, 30, couleur_polaris)
fleche(2000, xpol, ypol, 280, 15, couleur_polaris)
fleche(14000, xpol, ypol, 255, 35, couleur_polaris)

#  flèches d'annotation pour Vega :
fleche(14000, xveg, yveg, 120, 15, couleur_vega)
fleche(2000, xveg, yveg, 285, 35, couleur_vega)
fleche(9000, xveg, yveg, -5, 35, couleur_vega)
fleche(-12000, xveg, yveg, 150, 35, couleur_vega)

ax.set_xlabel('ascension droite')
ax.set_yticks(np.arange(0,60,10), minor=True)
ax.set_rlabel_position(45)  # position des graduations des rayons

ax.grid(which='major', alpha=0.6, linewidth=0.5)
# ax.grid(which='minor', alpha=0.4, linewidth=0.2)
ax.set_axisbelow(True)  # place la grille sous les points tracés.

ax.legend(loc="upper center")


#  signature :
fig.text(0.94, 0.05, "D. Alberto (www.astrolabe-science.fr) CC-BY-SA",
        fontsize=8, ha='right', va='bottom', rotation=90, c='gray',
        transform = fig.transFigure)

# =============================================================================
# EXPORT DES COORDONNEES CALCULEES :
# =============================================================================

df = pd.DataFrame({'annee': annees,
                   'ra_polaris_deg': np.degrees(xpol),
                   'dist_polaris_deg': ypol,
                   'ra_vega_deg': np.degrees(xveg),
                   'dist_vega_deg': yveg,
                   }                  )

# print(df)
#  export en fichier csv:
df.to_csv('positions_etoiles_precession.csv', sep='\t', index=False)

# =============================================================================
#
# =============================================================================

NomFichier="precession"
fig.savefig(NomFichier+".pdf")
fig.savefig(NomFichier+".png",dpi=300)
